/* $Id: Plasm.java,v 1.10 2005/10/08 11:20:57 ema_sab Exp $ */
package it.uniroma3.plasm.core;
import it.uniroma3.plasm.PlasmPlugin;
import it.uniroma3.plasm.preferences.PreferencePage;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.logging.Logger;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.ui.IDebugUIConstants;

/**
 * Class encapsulating the Plasm Process.
 * 
 * @version $Id: Plasm.java,v 1.10 2005/10/08 11:20:57 ema_sab Exp $
 * @author giupo
 */
public class Plasm {

	/**
	 * keeps the active state of this process
	 */

	private boolean active;

	/**
	 * Path to Plasm (executable included)
	 */
	private String plasmPath;

	/**
	 * The plasm process (JVM Standard API reference)
	 * Needed to obtain the standard input for the Plasm Process
	 * 
	 * @see java.lang.Process
	 */
	private Process pProcess;

	/**
	 * The plasm process (Eclipse API reference)
	 * Needed to monitor the Plasm process within the Eclipse Workbench
	 * @see java.lang.Process
	 */
	private IProcess plasmProcess;

	/**
	 * Plasm's standard output (where we outta read)
	 */
	private InputStream plasmStdOut;
	/**
	 * Plasm's error stream (where we wouldn't read, but we have to...)
	 */
	private InputStream plasmStdErr;
	/**
	 * Plasm's standard input (where we have to write)
	 */
	private OutputStream plasmStdIn;
	/**
	 * me, myself and I
	 * 
	 * @see The Singleton Pattern (google it)
	 */
	private static Plasm plasm;
	/**
	 * Prefix in the evaluation statement (schema stuff...)
	 */
	public static final byte[] PREFIX = "(eval-plasm \"".getBytes();
	/**
	 * Postfix in the evaluation statement. (Pretty dumb constant)
	 */
	public static final byte[] POSTFIX = "\")".getBytes();

	/**
	 * EXIT command for Plasm
	 */

	public static final byte[] EXIT = "(exit)".getBytes();

	public static final String NAME = "Plasm Process";

	private Logger log;
	
	/**
	 * Default constructor it is protected due to the singleton requirements it
	 * initializes the two threads addressed to read on the output streams of
	 * Plasm.
	 * 
	 * 2004/06/11 NOTE: no addictional threads anymore
	 * 
	 * The two Runnables are inner classes called Plasm$1 and Plasm$2 (Why have
	 * I written that?)
	 * 
	 * @param pit
	 *            the syncro class
	 */
	protected Plasm() {
		log = Logger.getLogger(getClass().getName());
		plasmPath = PlasmPlugin.getDefault().getPreferenceStore().getString(
				PreferencePage.PLASMFILE_PATH);
		;
	}
	/**
	 * method to retrieve the actual instance in the JVM See the <i>Singleton
	 * Pattern </i>
	 * 
	 * @param pit
	 *            the syncro class used to instantiate Plasm (without it there's
	 *            no reason to live)
	 */
	public static Plasm getInstance() {
		if (plasm == null) {
			plasm = new Plasm();
		}
		return plasm;
	}

	/**
	 * It opens, actually runs the plasm process and it connects the various
	 * streams to the crawling process.
	 * 
	 * @throws java.io.IOException
	 *             when it's impossible to communicate with the process.
	 */
	public void open() throws IOException {
		StringBuffer commandLine = new StringBuffer(plasmPath);

		//Controllo al volo che il file esista, altrimenti sollevo un eccezione
		File plasmFile = new File(plasmPath);
		if (!plasmFile.isFile()) {
			//Uh oh, something wrong here..that's not a file!
			throw new IOException("Plasm interpreter Not FOUND!");
		} else {
			/*
			 * simple reason: JVM holds the reference until the next GC call, we
			 * don't wanna hang the file system management... in this way we
			 * promote the plasmFile reference to be GC as soon as possible
			 */
			plasmFile = null;
		}

		//option FONDAMENTALE
		commandLine.append(" --dir ");
		int lastIndexOfSlash = plasmPath.lastIndexOf(System
				.getProperty("file.separator"));
		commandLine.append(plasmPath.substring(0, lastIndexOfSlash).replace('\\',
				'/'));
		log.warning(commandLine.toString());

		pProcess = Runtime.getRuntime().exec(commandLine.toString());

		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
ILaunchConfigurationType type = manager
.getLaunchConfigurationType("it.uniroma3.plasm.PlasmlaunchConfigurationType");

ILaunchConfigurationWorkingCopy launchWorkingCopy = null;
try {
	launchWorkingCopy = type.newInstance(null,
	"Run PLaSM");
} catch (CoreException e) {
	// TODO Auto-generated catch block
	e.printStackTrace();
}
launchWorkingCopy.setAttribute(IDebugUIConstants.ATTR_PRIVATE, true);
		ILaunch launch = new Launch(launchWorkingCopy, ILaunchManager.RUN_MODE, null);
		//manager.
		plasmProcess = DebugPlugin.newProcess(launch, pProcess, NAME);
		plasmStdOut = pProcess.getInputStream();
		plasmStdErr = pProcess.getErrorStream();
		plasmStdIn = pProcess.getOutputStream();
		//plasmProcess.setAttribute()
		
		manager.addLaunch(launch);
		active = true;

	}
	/**
	 * Ugly doodly doo, what this method is gonna do?
	 * 
	 * @throws java.lang.IOException
	 *             when somethings goes wrong closing all the streams (because
	 *             this is what this method does, if it's not straightforward to
	 *             your eyes...
	 */
	public void close() throws IOException {
		
		/* non controllo su isActive perche' ho bisogno di chiamare le close su tutti 
		 * metodi degli streams, anche se l'underlaying e' morto da un pezzo.
		 */ 
		
		if (active) {
			plasmStdIn.write(EXIT);
			plasmStdIn.flush();

			active = false;
			try {
				Thread.sleep(500);
				//To be sure, just in case
				if (!plasmProcess.isTerminated()) {
					log
							.warning("mmm, maybe something is hanging here, brute force terminate...");
					plasmProcess.terminate();
				}
			} catch (DebugException e) {
				log.warning(e.getMessage());
			} catch (InterruptedException e) {
				log.warning(e.getMessage());
			}
			plasmStdOut.close();
			plasmStdIn.close();
			plasmStdErr.close();
		}
	}

	/**
	 * AHCM (Another Hard comprehension method)
	 * 
	 * this method evaluates the string passed as argument to the running plasm
	 * process
	 * 
	 * NOTE: this method DOESN'T provide anysort of syntax evaluation...this is
	 * done from the underlying Plasm process.
	 * 
	 * @param code
	 *            the string containing the Plasm statements
	 * @throws java.io.IOException
	 *             when something odd (like broken communications) occurs.
	 */

	public void eval(String code) throws IOException {
		log.info(code);
		String prefix="(display \"\n";
		String postfix="\n\")";
		plasmStdIn.write(prefix.getBytes());
		plasmStdIn.write(code.getBytes());
		plasmStdIn.write(postfix.getBytes());
		plasmStdIn.write(PREFIX);
		plasmStdIn.write(code.getBytes());
		plasmStdIn.write(POSTFIX);
		plasmStdIn.flush();

		/*
		 * StringBuffer buffer = new StringBuffer(); buffer.append(PREFIX);
		 * buffer.append(code); buffer.append(POSTFIX);
		 * plasmProcess.getStreamsProxy().write(buffer.toString());
		 */

	}

	/**
	 * Is this process active?
	 * 
	 * @return Returns the active.
	 */
	public boolean isActive() {		
		return active && plasmProcess != null && !plasmProcess.isTerminated();
	}

	/**
	 * @return Returns the plasmPath.
	 */
	public String getPlasmPath() {
		return plasmPath;
	}
	/**
	 * Sets the plasm path NEEDED to run Plasm (as it takes it
	 * takes it as a parameter...)
	 * 
	 * @param plasmPath
	 *            The plasmPath to set.
	 */
	public void setPlasmPath(String plasmPath) {
		this.plasmPath = plasmPath;
	}

	public InputStream getPlasmStandardOutput() {
		return plasmStdOut;
	}
	
	/**
	 * Set a stream listener for this Plasm Process. I can have only ONE listener
	 * 
	 * @param listener
	 *        
	 * @throws IOException
	 *         Thrown if plasm is not active 
	 *
	 */
	public void setStreamListener(IStreamListener listener) throws IOException {
		if (!isActive()) {
			throw new IOException("Plasm is not active");
		}
		
		/*ho la garanzia che questa catena di metodi non tiri fuori una NullPointerException
		 dall'API di Eclipse (Quando ho un Processo, devo avere un Proxy, devo avere
		 uno StreamMonitor, e devo poter aggiungere un Listener, se cos? non fosse, ci 
		 sono altri problemi piu' grossi*/
		
		plasmProcess.getStreamsProxy().getOutputStreamMonitor().addListener(
				listener);
	}

}