package it.uniroma3.plasm.perspective;

import it.uniroma3.plasm.core.Plasm;
import it.uniroma3.plasm.views.PlasmView;

import java.io.IOException;
import java.util.logging.Logger;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

/**
 * Standard Perspective Class
 * 
 * This class provides the layout of the Plasm Perspective
 * 
 * @author giupo
 * @version $Id: PerspectiveFactory.java,v 1.4 2004/08/27 10:41:30 giupo Exp $
 * @see IPerspectiveFactory
 */
public class PerspectiveFactory implements IPerspectiveFactory {

	private Logger log;

	/**
	 * Standard constructor
	 */
	public PerspectiveFactory() {
		log = Logger.getLogger(this.getClass().getName());
	}

	/**
	 * Creates the initial layout for a page.
	 * <p>
	 * Implementors of this method may add additional views to a
	 * perspective.  The perspective already contains an editor folder
	 * identified by the result of <code>IPageLayout.getEditorArea()</code>.  
	 * Additional views should be added to the layout using this value as 
	 * the initial point of reference.  
	 * </p>
	 *
	 * @param layout the page layout
	 * 
	 * @see IPerspectiveFactory#createInitialLayout
	 */
	public void createInitialLayout(IPageLayout layout) {
		/*
		 * layout.addActionSet("ActionSet1");
		 * layout.addView("it.uniroma3.plasm.views.PlasmView",
		 * IPageLayout.BOTTOM, IPageLayout.DEFAULT_VIEW_RATIO, "");
		 * layout.addView("org.eclipse.ui.console.ConsoleView",
		 * IPageLayout.BOTTOM, IPageLayout.DEFAULT_FASTVIEW_RATIO, "");
		 * if(!layout.isEditorAreaVisible()) {
		 * layout.setEditorAreaVisible(true); log.warning("Editor area was not
		 * visible, restored to visible"); }
		 * log.warning(layout.getEditorArea());
		 */
		
		//		 Get the editor area.
		String editorArea = layout.getEditorArea();

		//Top left: Resource Navigator view and Bookmarks view placeholder
		IFolderLayout topLeft = layout.createFolder("topLeft",
				IPageLayout.LEFT, 0.25f, editorArea);
		topLeft.addView(IPageLayout.ID_RES_NAV);
		topLeft.addPlaceholder(IPageLayout.ID_BOOKMARKS);

		/* Bottom left: Outline view and Property Sheet view */
		IFolderLayout bottomLeft = layout.createFolder("bottomLeft",
				IPageLayout.BOTTOM, 0.50f, "topLeft");
		bottomLeft.addView(PlasmView.PART_ID);

		// this is necessary to force the Console on the BottomRight
		IFolderLayout bottomRight = layout.createFolder("bottomRight",
				IPageLayout.BOTTOM, 0.50f, editorArea);
		 
		//just in case Plasm in not active...
		try {
			Plasm p = Plasm.getInstance();
			if (!p.isActive()) {
				p.open();
			}
		} catch (IOException e) {
			log.info(e.toString());
		}
		// Bottom right: Task List view
		//layout.addView("it.uniroma3.plasm.views.PlasmView",
		// IPageLayout.BOTTOM, 0.66f, editorArea);
		//layout.addView("it.uniroma3.plasm.views.PlasmMessageConsole",
		// IPageLayout.BOTTOM, 0.66f, editorArea);
	}
}