package it.uniroma3.plasm.preferences;

import it.uniroma3.plasm.PlasmPlugin;
import it.uniroma3.plasm.core.Plasm;

import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This class represents a preference page that is contributed to the
 * Preferences dialog. By subclassing <samp>FieldEditorPreferencePage </samp>,
 * we can use the field support built into JFace that allows us to create a page
 * that is small and knows how to save, restore and apply itself. Cool!.
 * <p>
 * This page is used to modify preferences only. They are stored in the
 * preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store. 
 * (in a Singleton manner)
 * 
 * @author giupo
 * @version $Id: PreferencePage.java,v 1.5 2004/06/22 17:11:44 giupo Exp $
 */

public class PreferencePage extends FieldEditorPreferencePage
		implements
			IWorkbenchPreferencePage {

	public static final String PLASMFILE_PATH = "pathPreference";
	public static final String VRMLFILE_PATH = "vrmlFilePath";
	public static final String P_BOOLEAN = "booleanPreference";
	public static final String P_CHOICE = "choicePreference";
	public static final String P_STRING = "stringPreference";

	public PreferencePage() {
		super(GRID);
		setPreferenceStore(PlasmPlugin.getDefault().getPreferenceStore());
		setDescription("Plasm Preferences");
		initializeDefaults();
	}
	/**
	 * Sets the default values of the preferences.
	 */
	private void initializeDefaults() {
		IPreferenceStore store = getPreferenceStore();
		store.setDefault(P_BOOLEAN, true);
		store.setDefault(P_CHOICE, "choice2");
		store.setDefault(P_STRING, "Default value");
	}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */

	public void createFieldEditors() {
		addField(new FileFieldEditor(PLASMFILE_PATH,
				"Fully qualified &Plasm file:", getFieldEditorParent()));
		addField(new FileFieldEditor(VRMLFILE_PATH,
				"Fully qualified &VRML viewer:", getFieldEditorParent()));
		/*
		 * addField( new BooleanFieldEditor( P_BOOLEAN, "&An example of a
		 * boolean preference", getFieldEditorParent()));
		 */

		/*
		 * addField(new RadioGroupFieldEditor( P_CHOICE, "An example of a
		 * multiple-choice preference", 1, new String[][] { { "&Choice 1",
		 * "choice1" }, { "C&hoice 2", "choice2" } }, getFieldEditorParent()));
		 *  
		 */

		/*
		 * addField( new StringFieldEditor(P_STRING, "A &welcome message:",
		 * getFieldEditorParent()));
		 */
	}

	public void init(IWorkbench workbench) {

	}
	/**
	 * Method overridden to apply the change of the properties
	 */
	public boolean performOk() {
		boolean superReturned = super.performOk();
		updateState();
		return superReturned;
	}

	public void performApply() {
		super.performApply();
		updateState();
	}

	private void updateState() {
		Plasm plasm = Plasm.getInstance();
		IPreferenceStore store = PlasmPlugin.getDefault().getPreferenceStore();
		plasm.setPlasmPath(store.getString(PLASMFILE_PATH));
	}
}