package it.uniroma3.plasm.views;
import it.uniroma3.plasm.PlasmPlugin;
import it.uniroma3.plasm.preferences.PreferencePage;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
/**
 * The view shows data obtained from the model. The sample creates a dummy model 
 * on the fly, but a real implementation would connect to the model available either in
 * this or another plug-in (e.g. the workspace). The view is connected to the
 * model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be
 * presented in the view. Each view can present the same model objects using
 * different labels and icons, if needed. Alternatively, a single label provider
 * can be shared between views in order to ensure that objects of the same type
 * are presented in the same way everywhere.
 * <p>
 * 
 * <p>
 * This view actually shows the exports made from the plasm process, and it's notified
 * from the output poller associated to the process
 * 
 * @author giupo
 * @version $Id: PlasmView.java,v 1.15 2004/08/27 11:10:41 giupo Exp $
 */
public class PlasmView extends ViewPart {
	private TableViewer viewer;
	//private Action action1;
	//private Action action2;
	private Action doubleClickAction;
	private List model;
	private Logger log;
	protected boolean disposed;
	public final static String PART_ID = "it.uniroma3.plasm.views.PlasmView";
	private Process externalViewerProcess;
	/*
	 * The content provider class is responsible for providing objects to the
	 * view. It can wrap existing objects in adapters or simply return objects
	 * as-is. These objects may be sensitive to the current input of the view,
	 * or ignore it and always show the same content (like Task List, for
	 * example).
	 */
	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			return model.toArray();
		}
	}
	class ViewLabelProvider extends LabelProvider
			implements
				ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(obj);
		}
		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}
		public Image getImage(Object obj) {
			return PlatformUI.getWorkbench().getSharedImages().getImage(
					ISharedImages.IMG_OBJ_ELEMENT);
		}
	}
	/**
	 * The constructor.
	 */
	public PlasmView() {
		log = Logger.getLogger(this.getClass().getName());
		model = new ArrayList();
		disposed = false;
	}
	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		//viewer.setSorter(new NameSorter());
		viewer.setInput(getViewSite());
		viewer.setInput(model);
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
	}
	
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				PlasmView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}
	private void fillLocalPullDown(IMenuManager manager) {
		//manager.add(action1);
		//manager.add(new Separator());
		//manager.add(action2);
	}
	private void fillContextMenu(IMenuManager manager) {
		//manager.add(action1);
		//manager.add(action2);
		// Other plug-ins can contribute there actions here
		//manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	private void fillLocalToolBar(IToolBarManager manager) {
		//manager.add(action1);
		//manager.add(action2);
	}
	private void makeActions() {
		/*
		 * action1 = new Action() { public void run() { showMessage("Action 1
		 * executed"); } }; action1.setText("Action 1");
		 * action1.setToolTipText("Action 1 tooltip");
		 * action1.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
		 * .getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK)); action2 = new
		 * Action() { public void run() { showMessage("Action 2 executed"); } };
		 * action2.setText("Action 2"); action2.setToolTipText("Action 2
		 * tooltip");
		 * action2.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
		 * .getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
		 */
		doubleClickAction = new Action() {
			public void run() {
				String externalViewerCommand = PlasmPlugin.getDefault()
						.getPreferenceStore().getString(
								PreferencePage.VRMLFILE_PATH);
				if (externalViewerCommand == null || externalViewerCommand.length() == 0) {
					MessageDialog
							.openError(viewer.getControl().getShell(), "Plasm",
									"You have to set the Viewer in the Plasm Preferences Page");

					return;
				}
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection) selection)
						.getFirstElement();
				try {
					if(externalViewerProcess != null) {
						externalViewerProcess.destroy();
					}
					externalViewerProcess = Runtime.getRuntime().exec(
							externalViewerCommand + " " + obj.toString());
				} catch (IOException e) {
					MessageDialog.openError(viewer.getControl().getShell(),
							"Plasm", e.toString());
				}
			}
		};
	}
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	/**
	 * @return Returns the model.
	 */
	public List getModel() {
		return model;
	}
	/**
	 * @param model
	 *            The model to set.
	 */
	public void setModel(List model) {
		this.model = model;
	}

	/**
	 * WARNING : this is a blocking method, to be used inside a Display.async
	 * Call.
	 * 
	 * 
	 * Should we control the text highlight here? I think so...
	 * 
	 * another thing: should we synchronize the access to the Set?
	 * 
	 * @param obj
	 *            The FileName to be Added
	 */
	private void add(String obj) {
		if (model.contains(obj)) {
			log.info("il file name e' presente nella lista...");
			model.remove(obj);
		}
		model.add(0, obj);
		if (viewer != null) {
			viewer.refresh();
		} else {
			log.info("il viewer e' null");
		}
	}

	public void dispose() {
		super.dispose();
		log.info("called...");
		disposed = true;
	}

	public boolean isDisposed() {
		return disposed;
	}

	public void addAndUpdate(String fileName) {
		Display d = Display.getDefault();
		d.asyncExec(new Refresher(fileName));
	}

	/**
	 * delegate for the Viewer refresh
	 * 
	 * @see org.eclipse.jface.viewers.Viewer.refresh();
	 */

	public void refresh() {
		viewer.refresh();
	}

	public class Refresher implements Runnable {
		private Logger log;
		private String fileName;
		public Refresher(String fileName) {
			this.fileName = fileName;
			log = Logger.getLogger(this.getClass().getName());
		}
		public void run() {
			log.info("sto impartendo il refresh alla View");
			/*
			 * add ritorna un boolean se era gia' presente o no (vedere
			 * interfaccia Set) quindi si puo' usare per distinguere quando
			 * rievidenziare un filaname gia' presente
			 */
			add(fileName);
		}
	}
}