/* $Id: OutputPoller.java,v 1.15 2004/07/27 06:59:34 giupo Exp $ */
package it.uniroma3.plasm.core;
import it.uniroma3.plasm.views.PlasmView;

import java.util.logging.Logger;

import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.model.IStreamMonitor;
/**
 * This class provides a control on the output of the Plasm Process It controls,
 * through the IStreamListener, the outputstream and decides, concerning the
 * letterals coming out from it, some states of the application:
 * <ul>
 * <li>if Plasm has ended computing</li>
 * <li>if Plasm is exporting (and to command the refresh of the UI (very
 * tricky)</li>
 * </ul>
 * 
 * @author giupo
 * @version $Id: OutputPoller.java,v 1.15 2004/07/27 06:59:34 giupo Exp $
 */
public class OutputPoller implements IStreamListener {

	private final String PLASM_HAS_ENDED_STRING = "plasm> ";
	private final String PLASM_IS_EXPORTING_STRING = "Filename =";
	private final int PLASM_IS_EXPORTING_STRING_LENGTH = PLASM_IS_EXPORTING_STRING
			.length();
	protected final PlasmView view;
	private Logger log;
	
	
	/**
	 * Constructor
	 * 
	 * @param view
	 *      the view needed to add states if the output produces something 
	 *      influent ;)
	 */
	public OutputPoller(final PlasmView view) {
		this.view = view;
		log = Logger.getLogger(this.getClass().getName());
	}
	
	
	/**
	 * Notifies this listener that text has been appended to
	 * the given stream monitor.
	 *
	 * @param text the appended text
	 * @param monitor the stream monitor to which text was appended
	 
	 * @see org.eclipse.debug.core.IStreamListener
	 */
	public void streamAppended(String msg, IStreamMonitor monitor) {
		int beginningIndex = msg.indexOf(PLASM_IS_EXPORTING_STRING);
		if (beginningIndex != -1) {
			StringBuffer buffer = new StringBuffer(msg);
			while (buffer.length() > 0 && beginningIndex > -1) {

				//elimino tutto cio' che c'? prima della notazione di export
				buffer = buffer.delete(0, beginningIndex);

				int endLineIndex = buffer.indexOf("\n");
				String fileName = buffer.substring(
						PLASM_IS_EXPORTING_STRING_LENGTH, endLineIndex);
				log.info(fileName);

				view.addAndUpdate(fileName);
				buffer = buffer.delete(0, endLineIndex);
				beginningIndex = buffer.indexOf(PLASM_IS_EXPORTING_STRING);
			}
		}

		if (msg.endsWith(PLASM_HAS_ENDED_STRING)) {
			log.info("Plasm ha finito.");
		}
	}	
}