package it.uniroma3.plasm.editors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;

/**
 * A document partitioner divides a document into a set of disjoint text
 * partitions. This partitioner defines partition for PLaSM environment. In
 * PLaSM code exist only two type of regions: default and comment
 * 
 * @author EcT(o)PLaSM Group
 *  
 */
public class PlasmPartitionScanner extends RuleBasedPartitionScanner {

	/** Begin a list of partition to be used in PLaSM environment */
	public final static String COMMENT = "__comment";

	/**
	 * Contains all PLaSm partition identifiers used to
	 * create the partitioner
	 */
	public final static String[] PSM_PARTITION_TYPES = new String[]{
			IDocument.DEFAULT_CONTENT_TYPE, COMMENT};

	/**
	 * The constructor of PLaSM partition scanner. It contains all rules
	 * used to identify different partitions, till now is used only to identify
	 * PLaSM comments and default code
	 */
	public PlasmPartitionScanner() {

		/* rules is a list of all available partitions */
		List rules = new ArrayList();

		// TODO: Insert here all rule-based partition for PLaSM environment
		// Rule for multi-line comment, only one statement!
		IToken plasmComment = new Token(COMMENT);
		rules.add(new MultiLineRule("%", "%", plasmComment, (char) 0, true));

		IPredicateRule[] result = new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}
}