package it.uniroma3.plasm.editors.util;

import java.util.Arrays;

import org.eclipse.jface.text.rules.IWordDetector;

/**
 * Defines the detector by which WordRule determines 
 * whether a given character is valid as part of a word in the
 * current context. This implementation of the interface IWordDetector
 * stores all PLaSM operators. Since it applies only on single character 
 * operators, the isWordPart method return always false.
 * 
 * @see org.eclipse.jface.text.rules.IWordDetector
 * @author EcT(o)PLaSM Group
 */
public class PlasmOperatorDetector implements IWordDetector {
	
	/** This array contains all valid PLaSM operator */
	private static char[] operators = {'^', '-', '*', '/', '&', '#', '$', '+',
			'|', '~', ':', '=', '@', '!'};
	
	/**
	 * Implements the isWordStart method from IWordDetector interface. Checks
	 * if a character is valid as a first character of a PLaSM operator.
	 * 
	 * @param c the character to be checked
	 * @return true if the given character <code>c</code> is a PLaSM operator
	 * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
	 */
	public boolean isWordStart(char c) {
		Arrays.sort(operators);
		return Arrays.binarySearch(operators, c) >= 0;
	}
	
	/**
	 * Implements the isWordPart method from IWordDetector interface.
	 * Since PLaSM operators are single character words, this method is
	 * requested to return always false.
	 * 
	 * @param c the character to be checked
	 * @return false 
	 * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
	 */
	public boolean isWordPart(char c) {
		return false;
	}
	
}