package it.uniroma3.plasm.editors.util;

import java.util.Arrays;

import org.eclipse.jface.text.rules.IWordDetector;

/**
 * A PLaSM word detector for common words.
 *  
 * @author EcT(o)PLaSM Group
 */
public class PlasmWordDetector implements IWordDetector {
	
	/** Array of non alphanumeric characters admitted in PLaSM identifiers.
	 * Contains only the '_' character */
	private static char[] admittedCharacters = {'_'};
	
	/**
	 * Checks if a character is a valid PLaSM word initial character.
	 * 
	 * @param c the character to examine
	 * @return true if c is in the admittedCharacters array or is a letter,
	 * false otherwise 
	 * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
	 */
	public boolean isWordStart(char c) {
		
		boolean isStart = false;
		
		/* verify if is an admitted character */
		Arrays.sort(admittedCharacters);
		isStart |= Arrays.binarySearch(admittedCharacters, c) >= 0;
		
		/* verifiy if is a letter */
		isStart |= Character.isLetter(c);
		
		return isStart;
	}
	
	/**
	 * Checks if the given character c is a valid PLaSM internal word 
	 * character
	 * 
	 * @param c the character to check
	 * @return true if c is an alphanumeric character or is in the
	 * admittedCharacters array
	 * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
	 */
	public boolean isWordPart(char c) {
		return isWordStart(c) || Character.isDigit(c);
	}
	
}